/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2000-2001  HASEGAWA Masafumi
 */

#include <basic.h>
#include <bstdlib.h>
#include <bstdio.h>
#include <tcode.h>
#include <tctype.h>
#include <tstring.h>
#include <btron/btron.h>
#include <btron/libapp.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#define RECORD_TYPEMASK 0x80000000	// 쥳ɥ31
#define RECORD_SUBTYPE 0		// ֥0

#define MAX_EMULATE 20		// ߥ졼Ȥκ

// ߥ졼Ѿ
typedef struct
{
  W fd;		// եǥץ
  W size;	// 쥳ɥ 
  W offset;	// ߤɤ߹߰
}
BVFILE;

static BVFILE s_bvfile[MAX_EMULATE] = {
  {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
  {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}
};

// EUC-JPΥѥ̾TCѴ
static BOOL get_pathname (const char* pathname, TC* buffer)
{
  W len;

  len = eucstotcs (NULL, pathname);
  if (len >= L_PATHNM)
    {
      return FALSE;
    }
  eucstotcs (buffer, pathname);

  // եѥüʸѴ
  cnvpath(buffer);

  return TRUE;
}

// EUC-JPΥѥ̾LINK
WERR get_link (const char* pathname, LINK* link)
{
  TC buffer[L_PATHNM + 1];
  WERR err;

  // ȥǥ쥯ȥ꤫л
  if (pathname[0] == '.' && pathname[1] == '\0')
    {
      pathname += 1;
    }
  else if (pathname[0] == '.' && pathname[1] == '/')
    {
      pathname += 2;
    }
  if (!get_pathname (pathname, buffer))
    {
      return -1;
    }

  err = get_lnk (buffer, link, F_NORM);
  if (err < 0)
    {
      return err;
    }

  return err;
}

// ߥ졼Ⱦꤹ
static int set_file (W fd, W size)
{
  int i;

  for (i = 0; i < MAX_EMULATE; ++i)
    {
      if (s_bvfile[i].fd == -1)
	{
	  s_bvfile[i].fd = fd;
	  s_bvfile[i].size = size;
	  s_bvfile[i].offset = 0;
	  return i;
	}
    }

  return -1;
}

// ߥ졼Ⱦ
static BVFILE* get_file (int fd)
{
  if (fd < 0 || fd >= MAX_EMULATE)
    {
      return NULL;
    }
  if (s_bvfile[fd].fd == -1)
    {
      return NULL;
    }

  return s_bvfile + fd;
}

// open()ؿ򥨥ߥ졼Ȥ
int open (const char* pathname, int flags)
{
  LINK link;
  W mode, fd, size;
  WERR err;

  err = get_link (pathname, &link);
  if (err < 0)
    {
      return -1;
    }

  // ɤ߼ΤߥݡȤ
  mode = F_READ;
  fd = opn_fil (&link, mode, NULL);
  if (fd < 0)
    {
      return -1;
    }

  mode = F_TOPEND;
  err = fnd_rec (fd, mode, RECORD_TYPEMASK, RECORD_SUBTYPE, NULL);
  if (err < 0)
    {
      cls_fil (fd);
      return -1;
    }

  // 쥳ɥ
  err = rea_rec (fd, 0, NULL, 0, &size, NULL);
  if (err < 0)
    {
      cls_fil (fd);
      return -1;
    }

  return set_file (fd, size);
}

// close()ؿ򥨥ߥ졼Ȥ
int close (int fd)
{
  BVFILE *bvfile;
  WERR err;

  bvfile = get_file (fd);
  if (bvfile == NULL)
    {
      return -1;
    }

  err = cls_fil (bvfile->fd);
  if (err < 0)
    {
      return -1;
    }
  bvfile->fd = -1;

  return 0;
}

// read()ؿ򥨥ߥ졼Ȥ
ssize_t read (int fd, void* buffer, size_t size)
{
  BVFILE* bvfile;
  WERR err;

  bvfile = get_file (fd);
  if (bvfile == NULL)
    {
      return -1;
    }

  // 쥳ɥۤʤʤ褦Ĵ
  if (size > (size_t) (bvfile->size - bvfile->offset))
    {
      size = bvfile->size - bvfile->offset;
    }
  if (size <= 0)
    {
      return 0;
    }

  err = rea_rec (bvfile->fd, bvfile->offset, buffer, size, NULL, NULL);
  if (err < 0)
    {
      return -1;
    }
  bvfile->offset += size;

  return size;
}

// lseek()ؿ򥨥ߥ졼Ȥ
off_t lseek (int fd, off_t offset, int whence)
{
  BVFILE* bvfile;

  bvfile = get_file (fd);
  if (bvfile == NULL)
    {
      return -1;
    }

  switch (whence)
    {
    case SEEK_SET:
      bvfile->offset = offset;
      break;

    case SEEK_CUR:
      bvfile->offset += offset;
      break;

    case SEEK_END:
      bvfile->offset = bvfile->size - offset;
      break;
    }
  if (bvfile->offset < 0)
    {
      bvfile->offset = 0;
    }
  if (bvfile->offset > bvfile->size)
    {
      bvfile->offset = bvfile->size;
    }

  return bvfile->offset;
}

// stat()ؿ򥨥ߥ졼Ȥ
int stat (const char* pathname, struct stat* buffer)
{
  LINK link;
  F_STATE state;
  WERR err;

  if (get_link (pathname, &link) < 0)
    {
      return -1;
    }

  err = fil_sts (&link, NULL, &state, NULL);
  if (err < 0)
    {
      return -1;
    }
  if (buffer != NULL)
    {
      // եȥǥ쥯ȥΰ㤤ϤʤΤǡ
      // ե뤬¸ߤOKȤ
      buffer->st_mode = S_IFREG;
    }

  return 0;
}

// getcwd()ؿ򥨥ߥ졼Ȥ
char* getcwd (char* buffer, size_t size)
{
  if (buffer == NULL || size < 2)
    {
      return NULL;
    }

  // ȥǥ쥯ȥΥѥϼǤʤΤ
  // л֤Ƥget_link()
  buffer[0] = '.';
  buffer[1] = '\0';

  return buffer;
}


