/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001-2002  HASEGAWA Masafumi
 */

#ifndef __EBI_BOOK_H
#define __EBI_BOOK_H

#include "ebi_config.h"
#include "ebi_string.h"
#include "ebi_subbook.h"
#include "ebi_appendix.h"
#include "ebi_hookset.h"
#include "ebi_imagelink.h"

__EBI_BEGIN_NAMESPACE

class Fontset;
class TextPage;
class ImagePage;
class Wave;

class Book
{
 public:
  typedef enum
    {
      SEARCH_WORD,
      SEARCH_ENDWORD,
      SEARCH_EXACTWORD,
    }
  SearchMethod;
  enum
  {
    MAX_ENTRY = 50,
  };

  Book ();
  virtual ~Book ();

  static Book* findBook (EB_Book* book);
  bool changeWork ();
  bool open (const LINK & link);
  void close ();
  Hookset* getHookset ();
  Fontset* getFontset ();
  bool isCharCodeIso8859_1 ();
  bool getSubbookTitle (EB_Subbook_Code subbook, String* str);
  Subbook* getCurrSubbook ();
  Subbook* getSubbook (EB_Subbook_Code subbook) const;
  bool selectSubbook (EB_Subbook_Code subbook);
  bool haveCopyright ();
  bool getCopyright (EB_Position* pos);
  bool haveMenu ();
  bool getMenu (EB_Position* pos);
  bool getSubbookTitlePage (TextPage* page);
  bool getTextPage (const EB_Position& pos, TextPage* page);
  bool getEntryPage (const TC* word, SearchMethod search, TextPage* page);
  bool getEntryPageAll (const TC* word, SearchMethod search, TextPage* page);
  bool getImagePage (const EB_Position& pos, ImageLink::ImageType type, int width, int height, ImagePage* page);
  bool getSoundData (const EB_Position& pos, int size, Wave* wave);
#if 0
  bool getForwardPage (EB_Position* pos);
  bool getBackwardPage (EB_Position* pos);
#endif

 private:
  typedef int (*ReadFunction) (EB_Book*, EB_Appendix*, EB_Hookset*, void*, size_t, char*, ssize_t*);
  typedef std::map<EB_Book*, Book*> Books;

  static Books s_books;
  LINK m_link;
  EB_Book m_book;
  Subbooks m_subbooks;
  Appendix m_appendix;
  Hookset m_hookset;
  Fontset* m_fontset;
  TextPage* m_page;

  bool readString (ReadFunction function, const EB_Position & pos, std::string* str = NULL);
  bool searchWord (const TC * word, SearchMethod search);
};

__EBI_END_NAMESPACE

#endif /* __EBI_BOOK_H */
