/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_code.h"

__EBI_BEGIN_NAMESPACE

const TC Code::c_asciiTable[] = {
  TK_SP0, 0x212A, 0x2149, 0x2174, 0x2170, 0x2173, 0x2175, 0x2147,
  0x214A, 0x214B, 0x2176, 0x215C, 0x2124, 0x215D, 0x2125, 0x213F,
  0x2330, 0x2331, 0x2332, 0x2333, 0x2334, 0x2335, 0x2336, 0x2337,
  0x2338, 0x2339, 0x2127, 0x2128, 0x2163, 0x2161, 0x2164, 0x2129,
  0x2177, 0x2341, 0x2342, 0x2343, 0x2344, 0x2345, 0x2346, 0x2347,
  0x2348, 0x2349, 0x234A, 0x234B, 0x234C, 0x234D, 0x234E, 0x234F,
  0x2350, 0x2351, 0x2352, 0x2353, 0x2354, 0x2355, 0x2356, 0x2357,
  0x2358, 0x2359, 0x235A, 0x214E, 0x2140, 0x214F, 0x2130, 0x2132,
  0x212E, 0x2361, 0x2362, 0x2363, 0x2364, 0x2365, 0x2366, 0x2367,
  0x2368, 0x2369, 0x236A, 0x236B, 0x236C, 0x236D, 0x236E, 0x236F,
  0x2370, 0x2371, 0x2372, 0x2373, 0x2374, 0x2375, 0x2376, 0x2377,
  0x2378, 0x2379, 0x237A, 0x2150, 0x2143, 0x2151, 0x2131
};
const int Code::c_asciiSize = sizeof (c_asciiTable) / sizeof (TC);

const TC Code::c_latin1Table[] = {
  TK_SP1, 0x2923, 0x2171, 0x2172, 0x2924, 0x216F, 0x2925, 0x2178,
  0x212F, 0x2926, 0x2927, 0x2928, 0x224C, 0x213E, 0x292A, 0x292B, 
  0x216B, 0x215E, 0x292C, 0x292D, 0x216C, 0x264C, 0x2279, 0x292E, 
  0x292F, 0x2930, 0x2931, 0x2932, 0x2933, 0x2934, 0x2935, 0x2936, 
  0x2937, 0x2938, 0x2939, 0x293A, 0x293B, 0x293C, 0x293D, 0x293E, 
  0x293F, 0x2940, 0x2941, 0x2942, 0x2943, 0x2944, 0x2945, 0x2946, 
  0x2947, 0x2948, 0x2949, 0x294A, 0x294B, 0x294C, 0x294D, 0x2160, 
  0x294E, 0x294F, 0x2950, 0x2951, 0x2952, 0x2953, 0x2954, 0x2955, 
  0x2956, 0x2957, 0x2958, 0x2959, 0x295A, 0x295B, 0x295C, 0x295D, 
  0x295E, 0x295F, 0x2960, 0x2961, 0x2962, 0x2963, 0x2964, 0x2965, 
  0x2966, 0x2967, 0x2968, 0x2969, 0x296A, 0x296B, 0x296C, 0x215F, 
  0x296D, 0x296E, 0x296F, 0x2970, 0x2971, 0x2972, 0x2973, 0x2974
};
const int Code::c_latin1Size = sizeof (c_latin1Table) / sizeof (TC);

Code::Code ()
{
}

TC Code::asciiToTron (UB src)
{
  if (src < 0x20 || src > 0x7E)
    {
      return TK_NULL;
    }
  return c_asciiTable[src - 0x20];
}

TC Code::latin1ToTron (UB src)
{
  if (!(src & 0x80))
    {
      return asciiToTron (src);
    }
  src &= 0x7F;

  if (src < 0x20 || src > 0x7F)
    {
      return TK_NULL;
    }
  return c_latin1Table[src - 0x20];
}

TC Code::eucJpToTron (UH src)
{
  return (src & 0x7F7F);
}

TC Code::eucGbToTron (UH src)
{
  UH ku = ((src >> 8) & 0x7F) - 0x20;
  UH ten = (src & 0x7F) - 0x20;
  UH no = (ku - 1) * 94 + (ten - 1);
  return (((no / 126) + 0x21) << 8) + ((no % 126) + 0x80);
}

void Code::tronToEucJp (const TC * src, std::string * dest)
{
  W length = tcstoeucs (NULL, src);
  if (length == -1)
    {
      return;
    }

  char *str = new char[length + 1];
  tcstoeucs (str, src);
  *dest = str;
  delete[]str;
}

void Code::eucJpToTron (const char *src, String * dest)
{
  W length = eucstotcs (NULL, src);
  if (length == -1)
    {
      return;
    }

  TC *str = new TC[length + 1];
  eucstotcs (str, src);
  *dest = str;
  delete[]str;
}

void Code::tronToLatin1 (const TC * src, std::string* dest)
{
  dest->erase ();
  for (; *src != TK_NULL; ++src)
    {
      UB ch = 0;
      for (int i = 0; i < c_asciiSize; ++i)
	{
	  if (*src == c_asciiTable[i])
	    {
	      ch = (UB) (i + 0x20);
	    }
	}
      if (ch == 0)
	{
	  for (int i = 0; i < c_latin1Size; ++i)
	    {
	      if (*src == c_latin1Table[i])
		{
		  ch += (UB) (i + 0xA0);
		}
	    }
	}
      *dest += ((char) ch);
    }
}

void Code::latin1ToTron (const char *src, String* dest)
{
  dest->erase ();
  for (; *src != '\0'; ++src)
    {
      *dest += latin1ToTron (*src);
    }
}

bool Code::isLatin1 (TC code)
{
  const TC* ptr = std::find (c_asciiTable, c_asciiTable + c_asciiSize, code);
  if (ptr == NULL)
    {
      ptr = std::find (c_latin1Table, c_latin1Table + c_latin1Size, code);
      if (ptr == NULL)
	{
	  return false;
	}
    }
  return true;
}

__EBI_END_NAMESPACE
