/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#ifndef __EBI_DEVICE_H
#define __EBI_DEVICE_H

#include "ebi_config.h"
#include "ebi_global.h"
#include "ebi_container.h"

__EBI_BEGIN_NAMESPACE

class BitmapFont;

class Device
{
 public:
  Device (GID gid);
  virtual ~Device ();

 public:
  W getFontSize () const;
  void setFontSize (W size);
  UW getFontAttr () const;
  void setFontAttr (UW attr);
  void updateFontSpec ();
  void selectFont (FontType fontType);
  W getCharWidth (TC code) const;
  void drawChar (TC code);
  void drawBitmapFont (BitmapFont* bitmapFont, const SIZE& size);
  void newPage ();
  void newLine ();
  void beginLink ();
  void endLink ();
  W calcLines () const;
  W calcMax () const;
  PNT calcPoint (const PNT& pnt, W low) const;
  void setRect (const RECT& rect);
  void clear ();
  GID getGid () const;
  W getMargin () const;
  W getLineFeed () const;

 private:
  static const COLOR c_textColor;
  static const COLOR c_linkColor;
  static const COLOR c_backColor;

  GID m_gid;
  RECT m_rect;
  W m_fontSize;
  UW m_fontAttr;
  FontType m_fontType;
  FSSPEC* m_font;
  FSSPEC m_halfFont;
  FSSPEC m_fullFont;
  COLOR m_color;
};
  
__EBI_END_NAMESPACE

#endif /* __EBI_DEVICE_H */
