/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_fontset.h"
#include "ebi_bitmapfont.h"
#include "ebi_error.h"
#include "ebi_element.h"

__EBI_BEGIN_NAMESPACE

Fontset::Fontset (EB_Book * book, EB_Font_Code fontCode)
{
  m_book = book;
  m_fontCode = fontCode;
}

Fontset::~Fontset ()
{
  clear ();
}

void Fontset::clear ()
{
  for (Fonts::iterator fi = m_narrowFonts.begin (); fi != m_narrowFonts.end (); ++fi)
    {
      if ((*fi).second != NULL)
	{	
	  delete (*fi).second;
	}
    }
  m_narrowFonts.clear ();

  for (Fonts::iterator fi = m_wideFonts.begin (); fi != m_wideFonts.end (); ++fi)
    {
      if ((*fi).second != NULL)
	{	
	  delete (*fi).second;
	}
    }
  m_wideFonts.clear ();
}

BitmapFont *Fontset::getBitmapFont (EB_Subbook_Code subbook, int code, FontType fontType)
{
  W height;
  Fonts *fonts = NULL;
  SIZE size;
  EB_Error_Code (*func) (EB_Book *, int, char *) = NULL;
  eb_font_height2 (m_fontCode, &height);

  switch (fontType)
    {
    case FONT_HALF:
      fonts = &m_narrowFonts;
      size.h = height / 2;
      size.v = height;
      func = eb_narrow_font_character_bitmap;
      break;

    case FONT_FULL:
      fonts = &m_wideFonts;
      size.h = height;
      size.v = height;
      func = eb_wide_font_character_bitmap;
      break;
      
    default:
      return NULL;
    }

  Fonts::iterator fi = fonts->find (code);
  if (fi != fonts->end ())
    {
      return (*fi).second;
    }

  Error::s_error = eb_set_subbook (m_book, subbook);
  if (Error::s_error != EB_SUCCESS)
    {
      Error::outputMessageEB ("ܤǤޤ");
      return NULL;
    }

  Error::s_error = eb_set_font (m_book, m_fontCode);
  if (Error::s_error != EB_SUCCESS)
    {
      Error::outputMessageEB ("եȤǤޤ");
      return NULL;
    }

  char binary[EB_SIZE_WIDE_FONT_48];
  Error::s_error = (*func) (m_book, code, binary);
  if (Error::s_error != EB_SUCCESS)
    {
      Error::outputMessageEB ("եȤǤޤ");
      return NULL;
    }

  (*fonts)[code] = new BitmapFont (size, binary);

  return (*fonts)[code];
}

__EBI_END_NAMESPACE
