/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_fusen.h"
#include "ebi_error.h"

__EBI_BEGIN_NAMESPACE

Fusen::Fusen ()
{
  m_data.size = sizeof (m_data) - sizeof (UH);
  m_data.dummy1 = 0;
  m_data.dummy2 = 0;
  m_data.dummy3 = 0;
  m_data.search = Book::SEARCH_WORD;
  m_data.bFullview = false;
  m_data.rect = toRECT (100, 40, 540, 400);
}

Fusen::~Fusen ()
{
}

Fusen::Data * Fusen::readFile (M_EXECREQ * exec, Data * data)
{
  W fd, err;

  fd = opn_fil (&exec->lnk, F_READ, NULL);
  if (fd < 0)
    {
      Error::outputMessageBV ("opn_fil", fd);
      return NULL;
    }
  err = see_rec (fd, exec->info, 1, NULL);
  if (err < 0)
    {
      Error::outputMessageBV ("see_rec", err);
      cls_fil (fd);
      return NULL;
    }
  err = rea_rec (fd, 94, (B *) data, sizeof (Data), NULL, NULL);
  cls_fil (fd);

  if (err < 0)
    {
      Error::outputMessageBV ("rea_rec", err);
      return NULL;
    }

  return data;
}

void Fusen::read (MESSAGE * msg)
{
  Data data;
  Data *info = NULL;
  switch (msg->msg_type)
    {
    case EXECREQ:
      {
	M_EXECREQ *exec = (M_EXECREQ *) msg;
	if (exec->mode & 0x0001)
	  {
	    info = readFile (exec, &data);
	  }
	else
	  {
	    info = (Data *) exec->info;
	  }
      }
      break;

    case FUSENREQ:
      info = (Data *) ((M_FUSENREQ *) msg)->info;
      break;

    case DISPREQ:
      info = (Data *) ((M_DISPREQ *) msg)->info;
      break;

    case PASTEREQ:
      info = (Data *) ((M_PASTEREQ *) msg)->info;
      break;
    }

  if (info != NULL && info->size == m_data.size)
    {
      m_data = *info;
    }
}

void* Fusen::write (MESSAGE* msg)
{
  LINK* lnk = NULL;
  W vid = 0;
  switch (msg->msg_type)
    {
    case EXECREQ:
      {
	M_EXECREQ* exec = (M_EXECREQ*) msg;
	lnk = &exec->lnk;
	vid = exec->vid;
      }
      break;

    case DISPREQ:
      {
	M_DISPREQ* disp = (M_DISPREQ*) msg;
	lnk = &disp->lnk;
	vid = disp->vid;
      }
      break;

    case PASTEREQ:
      {
	M_PASTEREQ* paste = (M_PASTEREQ*) msg;
	lnk = &paste->lnk;
	vid = paste->vid;
      }
      break;

    case FUSENREQ:
    default:
      return NULL;
    }

  W fd = opn_fil (lnk, F_UPDATE, NULL);
  if (fd < 0)
    {
      Error::outputMessageBV ("opn_fil", fd);
      return NULL;
    }
  ERR err = oput_fsn (vid, fd, &m_data);
  cls_fil (fd);

  if (err < 0)
    {
      Error::outputMessageBV ("oput_fsn", err);
      return NULL;
    }

  return &m_data;
}

bool Fusen::getFullview () const
{
  return m_data.bFullview;
}

void Fusen::setFullview (bool bFullview)
{
  m_data.bFullview = bFullview;
}

const RECT & Fusen::getWindowRect () const
{
  return m_data.rect;
}

void Fusen::setWindowRect (const RECT & rect)
{
  m_data.rect = rect;
}

Book::SearchMethod Fusen::getSearchMethod () const
{
  return (Book::SearchMethod) m_data.search;
}

void Fusen::setSearchMethod (Book::SearchMethod search)
{
  m_data.search = search;
}

__EBI_END_NAMESPACE
