/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001-2002  HASEGAWA Masafumi
 */

#include "ebi_hookset.h"
#include "ebi_book.h"
#include "ebi_pagemaker.h"
#include "ebi_element.h"
#include "ebi_error.h"
#include "ebi_code.h"
#include "ebi_imagelink.h"

__EBI_BEGIN_NAMESPACE

const EB_Hook Hookset::s_hooks[] =
{
  {EB_HOOK_NEWLINE, hookFunc},
  {EB_HOOK_BEGIN_NO_NEWLINE, hookFunc},
  {EB_HOOK_END_NO_NEWLINE, hookFunc},
  {EB_HOOK_BEGIN_REFERENCE, hookFunc},
  {EB_HOOK_END_REFERENCE, hookFunc},
  {EB_HOOK_BEGIN_CANDIDATE, hookFunc},
  {EB_HOOK_END_CANDIDATE_LEAF, hookFunc},
  {EB_HOOK_END_CANDIDATE_GROUP, hookFunc},

  {EB_HOOK_NARROW_JISX0208, hookFunc},
  {EB_HOOK_WIDE_JISX0208, hookFunc},
  {EB_HOOK_ISO8859_1, hookFunc},
  {EB_HOOK_GB2312, hookFunc},
  {EB_HOOK_NARROW_FONT, hookFunc},
  {EB_HOOK_WIDE_FONT, hookFunc},

  {EB_HOOK_SET_INDENT, hookFunc},
  {EB_HOOK_BEGIN_EMPHASIS, hookFunc},
  {EB_HOOK_END_EMPHASIS, hookFunc},
  {EB_HOOK_BEGIN_KEYWORD, hookFunc},
  {EB_HOOK_END_KEYWORD, hookFunc},
  {EB_HOOK_BEGIN_SUPERSCRIPT, hookFunc},
  {EB_HOOK_END_SUPERSCRIPT, hookFunc},
  {EB_HOOK_BEGIN_SUBSCRIPT, hookFunc},
  {EB_HOOK_END_SUBSCRIPT, hookFunc},

  {EB_HOOK_BEGIN_MONO_GRAPHIC, hookFunc},
  {EB_HOOK_END_MONO_GRAPHIC, hookFunc},
  {EB_HOOK_BEGIN_GRAY_GRAPHIC, hookFunc},
  {EB_HOOK_END_GRAY_GRAPHIC, hookFunc},
  {EB_HOOK_BEGIN_COLOR_BMP, hookFunc},
  {EB_HOOK_BEGIN_COLOR_JPEG, hookFunc},
  {EB_HOOK_END_COLOR_GRAPHIC, hookFunc},

  {EB_HOOK_END_IN_COLOR_GRAPHIC, hookFunc},
  {EB_HOOK_BEGIN_WAVE, hookFunc},
  {EB_HOOK_END_WAVE, hookFunc},
  {EB_HOOK_BEGIN_MPEG, hookFunc},
  {EB_HOOK_END_MPEG, hookFunc},

  {EB_HOOK_BEGIN_GRAPHIC_REFERENCE, hookFunc},
  {EB_HOOK_END_GRAPHIC_REFERENCE, hookFunc},
  {EB_HOOK_GRAPHIC_REFERENCE, hookFunc},

  {EB_HOOK_NULL, NULL}
};

Hookset::Hookset ()
{
  m_pageMaker = NULL;

  eb_initialize_hookset (&m_hookset);
  eb_set_hooks (&m_hookset, s_hooks);
}

Hookset::~Hookset ()
{
}

EB_Hookset *Hookset::getHookset ()
{
  return &m_hookset;
}

void Hookset::setPageMaker (PageMaker * pageMaker)
{
  m_pageMaker = pageMaker;
}

EB_Error_Code Hookset::hookFunc (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  //Error::outputMessageHook (code);

  Hookset *hookset = Book::findBook (book)->getHookset ();
  switch (code)
    {
    case EB_HOOK_NEWLINE:
      return hookset->hookNewLine (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_NO_NEWLINE:
      return hookset->hookBeginNoNewLine (book, ptr, argc, argv);

    case EB_HOOK_END_NO_NEWLINE:
      return hookset->hookEndNoNewLine (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_REFERENCE:
      return hookset->hookBeginReference (book, ptr, argc, argv);

    case EB_HOOK_END_REFERENCE:
      return hookset->hookEndReference (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_CANDIDATE:
      return hookset->hookBeginCandidate (book, ptr, argc, argv);

    case EB_HOOK_END_CANDIDATE_LEAF:
      return hookset->hookEndCandidateLeaf (book, ptr, argc, argv);

    case EB_HOOK_END_CANDIDATE_GROUP:
      return hookset->hookEndCandidateGroup (book, ptr, argc, argv);

    case EB_HOOK_NARROW_JISX0208:
      return hookset->hookNarrowJisx0208 (book, ptr, argc, argv);

    case EB_HOOK_WIDE_JISX0208:
      return hookset->hookWideJisx0208 (book, ptr, argc, argv);

    case EB_HOOK_ISO8859_1:
      return hookset->hookIso8859_1 (book, ptr, argc, argv);

    case EB_HOOK_GB2312:
      return hookset->hookGb2312 (book, ptr, argc, argv);

    case EB_HOOK_NARROW_FONT:
      return hookset->hookNarrowFont (book, ptr, argc, argv);

    case EB_HOOK_WIDE_FONT:
      return hookset->hookWideFont (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_MONO_GRAPHIC:
      return hookset->hookBeginMonoGraphic (book, ptr, argc, argv);

    case EB_HOOK_END_MONO_GRAPHIC:
      return hookset->hookEndMonoGraphic (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_GRAY_GRAPHIC:
      return hookset->hookBeginGrayGraphic (book, ptr, argc, argv);

    case EB_HOOK_END_GRAY_GRAPHIC:
      return hookset->hookEndGrayGraphic (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_COLOR_BMP:
      return hookset->hookBeginColorBmp (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_COLOR_JPEG:
      return hookset->hookBeginColorJpeg (book, ptr, argc, argv);

    case EB_HOOK_END_COLOR_GRAPHIC:
      return hookset->hookEndColorGraphic (book, ptr, argc, argv);

    case EB_HOOK_BEGIN_WAVE:
      return hookset->hookBeginWave (book, ptr, argc, argv);

    case EB_HOOK_END_WAVE:
      return hookset->hookEndWave (book, ptr, argc, argv);

    case EB_HOOK_SET_INDENT:
    case EB_HOOK_BEGIN_EMPHASIS:
    case EB_HOOK_END_EMPHASIS:
    case EB_HOOK_BEGIN_KEYWORD:
    case EB_HOOK_END_KEYWORD:
    case EB_HOOK_BEGIN_SUPERSCRIPT:
    case EB_HOOK_END_SUPERSCRIPT:
    case EB_HOOK_BEGIN_SUBSCRIPT:
    case EB_HOOK_END_SUBSCRIPT:
    case EB_HOOK_END_IN_COLOR_GRAPHIC:
    case EB_HOOK_BEGIN_MPEG:
    case EB_HOOK_END_MPEG:
    case EB_HOOK_BEGIN_GRAPHIC_REFERENCE:
    case EB_HOOK_END_GRAPHIC_REFERENCE:
    case EB_HOOK_GRAPHIC_REFERENCE:
      // TODO: HOOK
      break;
    }
  return 0;
}

EB_Error_Code Hookset::hookNewLine (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->newLine ();
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginNoNewLine (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->beginWord ();
    }

  return 0;
}

EB_Error_Code Hookset::hookEndNoNewLine (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->endWord ();
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginReference (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->beginReference (Book::findBook (book)->getCurrSubbook ());
    }

  return 0;
}

EB_Error_Code Hookset::hookEndReference (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[1], argv[2]
  };
  if (m_pageMaker)
    {
      m_pageMaker->endReference (pos);
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginCandidate (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  return hookBeginReference (book, container, argc, argv);
}

EB_Error_Code Hookset::hookEndCandidateLeaf (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  // TODO: HOOK_END_CANDIDATE_LEAF
  return hookEndReference (book, container, argc, argv);
}

EB_Error_Code Hookset::hookEndCandidateGroup (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  return hookEndReference (book, container, argc, argv);
}

EB_Error_Code Hookset::hookNarrowJisx0208 (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  TC code = Code::eucJpToTron (argv[0]);
  if (m_pageMaker)
    {
      m_pageMaker->putChar (code, FONT_HALF);
    }

  return 0;
}

EB_Error_Code Hookset::hookWideJisx0208 (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  TC code = Code::eucJpToTron (argv[0]);
  if (m_pageMaker)
    {
      m_pageMaker->putChar (code, FONT_FULL);
    }

  return 0;
}

EB_Error_Code Hookset::hookIso8859_1 (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  TC code = Code::latin1ToTron (argv[0] & 0xFF);
  if (m_pageMaker)
    {
      m_pageMaker->putChar (code, FONT_HALF);
    }

  return 0;
}

EB_Error_Code Hookset::hookGb2312 (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  TC code = Code::eucGbToTron (argv[0]);
  if (m_pageMaker)
    {
      m_pageMaker->putChar (code, FONT_FULL);
    }

  return 0;
}

EB_Error_Code Hookset::hookNarrowFont (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  int code = argv[0];
  if (m_pageMaker)
    {
      m_pageMaker->putGaiji (Book::findBook (book)->getCurrSubbook (), code, FONT_HALF);
    }

  return 0;
}

EB_Error_Code Hookset::hookWideFont (EB_Book * book, void* ptr, int argc, const unsigned int *argv)
{
  int code = argv[0];
  if (m_pageMaker)
    {
      m_pageMaker->putGaiji (Book::findBook (book)->getCurrSubbook (), code, FONT_FULL);
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginMonoGraphic (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  int width = argv[3];
  int height = argv[2];
  if (m_pageMaker)
    {
      m_pageMaker->beginGraphic (Book::findBook (book)->getCurrSubbook (), ImageLink::TYPE_MONO, width, height);
    }

  return 0;
}

EB_Error_Code Hookset::hookEndMonoGraphic (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[1], argv[2]
  };
  if (m_pageMaker)
    {
      m_pageMaker->endGraphic (pos);
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginGrayGraphic (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  int width = argv[3];
  int height = argv[2];
  if (m_pageMaker)
    {
      m_pageMaker->beginGraphic (Book::findBook (book)->getCurrSubbook (), ImageLink::TYPE_GRAY, width, height);
    }

  return 0;
}

EB_Error_Code Hookset::hookEndGrayGraphic (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[1], argv[2]
  };
  if (m_pageMaker)
    {
      m_pageMaker->endGraphic (pos);
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginColorBmp (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[2], argv[3]
  };
  if (m_pageMaker)
    {
      m_pageMaker->beginGraphic (Book::findBook (book)->getCurrSubbook (), ImageLink::TYPE_COLOR, pos);
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginColorJpeg (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[2], argv[3]
  };
  if (m_pageMaker)
    {
      m_pageMaker->beginGraphic (Book::findBook (book)->getCurrSubbook (), ImageLink::TYPE_JPEG, pos);
    }

  return 0;
}

EB_Error_Code Hookset::hookEndColorGraphic (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->endGraphic ();
    }

  return 0;
}

EB_Error_Code Hookset::hookBeginWave (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  EB_Position pos = {
    argv[2], argv[3]
  };
  int start = (argv[2] - 1) * EB_SIZE_PAGE + argv[3];
  int end = (argv[4] - 1) * EB_SIZE_PAGE + argv[5];
  int size = end - start;
  if (m_pageMaker)
    {
      m_pageMaker->beginSound (Book::findBook (book)->getCurrSubbook (), pos, size);
    }

  return 0;
}

EB_Error_Code Hookset::hookEndWave (EB_Book* book, void* container, int argc, const unsigned int* argv)
{
  if (m_pageMaker)
    {
      m_pageMaker->endSound ();
    }

  return 0;
}

__EBI_END_NAMESPACE
