/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_imagepage.h"
#include "ebi_error.h"
#include "ebi_device.h"

__EBI_BEGIN_NAMESPACE

ImagePage::ImagePage (Subbook* subbook, const EB_Position* pos)
  : Page (subbook, pos)
{
  m_binary = NULL;
  m_length = 0;
}

ImagePage::~ImagePage ()
{
  clear ();
}

void ImagePage::clear ()
{
  if (m_binary != NULL)
    {
      free (m_binary);
      m_binary = NULL;
      m_length = 0;
    }
}

bool ImagePage::addBinary (const char* binary, int length)
{
  if (m_binary == NULL)
    {
      m_binary = (char*) malloc (length);
      if (m_binary == NULL)
	{
	  return false;
	}
      memcpy (m_binary, binary, length);
      m_length = length;
    }
  else
    {
      char* temp = (char*) realloc (m_binary, m_length + length);
      if (temp == NULL)
	{
	  clear ();
	  return false;
	}
      m_binary = temp;
      memcpy (m_binary + m_length, binary, length);
      m_length += length;
    }

  return true;
}

bool ImagePage::convertBinary (ImageLink::ImageType type)
{
  bool result = false;
  switch (type)
    {
    case ImageLink::TYPE_MONO:
    case ImageLink::TYPE_GRAY:
    case ImageLink::TYPE_COLOR:
      result = m_bitmap.convertBitmap (m_binary, m_length);
      break;

    case ImageLink::TYPE_JPEG:
      result = m_bitmap.convertJpeg (m_binary, m_length);
      break;
    }

  return result;
}

void ImagePage::draw (W low, W high)
{
  if (m_device == NULL)
    {
      return;
    }
  GID gid = m_device->getGid ();

  W x = m_device->getMargin ();
  W y = m_device->getMargin ();
  m_bitmap.draw (gid, x, y);
}

W ImagePage::updateLayout ()
{
  if (m_device == NULL)
    {
      return 0;
    }

  return 1;
}

const Reference* ImagePage::getReference (const PNT& pnt) const
{
  return NULL;
}

__EBI_END_NAMESPACE
