/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_config.h"
#include "ebi.h"
#include "ebi_error.h"
#include "ebi_textpage.h"
#include "ebi_imagepage.h"
#include "ebi_menu.h"
#include "ebi_reference.h"
#include "databox.h"

__EBI_BEGIN_NAMESPACE

#define APP_PICT_ID 10

static union
{
  MESSAGE *msg;
  M_EXECREQ *exec;
  M_DISPREQ *disp;
  M_PASTEREQ *paste;
  M_FUSENREQ *fusen;
  M_TADREQ *tad;
}
s_msg;

Fusen g_fusen;
MainWindow g_window;
Menu g_menu;
Book g_book;
PageSet g_pageSet;

static W s_targetvid = -1;

void openSubbookTitlePage ()
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  TextPage* page = new TextPage;
  if (g_book.getSubbookTitlePage (page))
    {
      g_pageSet.insert (page);
    }
  else
    {
      delete page;
    }
}

void openReferencePage (const Reference* reference)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);
      
  Subbook* subbook = reference->getSubbook ();
  Book* book = subbook->getBook ();
  if (book->selectSubbook (subbook->getSubbook ()))
    {
      Page* page = reference->execute ();
      if (page != NULL)
	{
	  g_pageSet.insert (page);
	}
    }
}

extern "C" void MAIN (MESSAGE * msg)
{
  W pwid = 0;
  LINK targetlnk;
  LINK selflnk;
  s_msg.msg = msg;
  switch (s_msg.msg->msg_type)
    {
    case EXECREQ:
      if ((s_msg.exec->mode & 0x0002) != 0)
	{
	  /* ʪư */
	  char str[40];
	  sprintf (str, "EXECREQ mode=%08x", s_msg.exec->mode);
	  Error::outputMessage (str);
	  ext_prc (-1);
	}

      targetlnk = s_msg.exec->lnk;
      selflnk = s_msg.exec->self;
      s_targetvid = s_msg.exec->vid;
      pwid = s_msg.exec->pwid;
      break;

    case FUSENREQ:
      Error::outputMessage ("DISPREQ");
      oend_req (s_msg.fusen->vid, -1);
      ext_prc (-1);
      break;

    case DISPREQ:
      Error::outputMessage ("DISPREQ");
      oend_req (s_msg.disp->vid, -1);
      ext_prc (-1);

    case PASTEREQ:
      Error::outputMessage ("PASTEREQ");
      oend_req (s_msg.paste->vid, -1);
      ext_prc (-1);

    case TADREQ:
      Error::outputMessage ("TADREQ");
      oend_req (s_msg.tad->vid, -1);
      ext_prc (-1);

    default:
      {
	char str[40];
	sprintf (str, "msg_type=%d", s_msg.msg->msg_type);
	Error::outputMessage (str);
      }
      ext_prc (-1);
    }

  ERR err = opendbox (&selflnk, USER_DTYP, USER_DNUM);
  if (err != E_OK)
    {
      Error::outputMessageBV ("opendbox", err);
      ext_prc (-1);
    }
  getscreen ();
  initstdpnl ();

  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);
  g_fusen.read (s_msg.msg);

  TC titleName[L_FNM + 1];
  err = fil_sts (&targetlnk, titleName, NULL, NULL);
  if (err < 0)
    {
      Error::outputMessageBV ("fil_sts", err);
      tc_strcpy (titleName, getdbox_text (TX_APP_NAME));
    }
  W wid = g_window.open (pwid, APP_PICT_ID, titleName);
  if (wid < 0)
    {
      killMe (1);
    }
  g_menu.open ();

  switch (s_msg.msg->msg_type)
    {
    case EXECREQ:
      osta_prc (s_targetvid, wid);
      break;
    }

  if (!g_book.open (targetlnk))
    {
      killMe (1);
    }

  openSubbookTitlePage ();

  W exitCode = Window::eventLoop ();
  killMe (exitCode);
}

void killMe (W exitCode)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) - 1, (COLOR) - 1);

  VP dat = NULL;
  if (exitCode > 0)
    {
      dat = g_fusen.write (s_msg.msg);
    }
  if (s_targetvid > 0)
    {
      oend_prc (s_targetvid, dat, 1);
    }

  g_book.close ();
  g_window.close ();
  closemenu ();
  closedbox ();

  ext_prc (0);
}

__EBI_END_NAMESPACE
