/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#ifndef __EBI_PAGEMAKER_H
#define __EBI_PAGEMAKER_H

#include "ebi_config.h"
#include "ebi_string.h"
#include "ebi_global.h"
#include "ebi_imagelink.h"

__EBI_BEGIN_NAMESPACE

class Subbook;
class TextPage;
class Line;
class Reference;
class Word;
class Element;

class PageMaker
{
 public:
  PageMaker (TextPage * page);
  virtual ~PageMaker ();

  void newLine ();
  void undoLine ();
  void beginReference (Subbook* subbook);
  void endReference (const EB_Position & pos);
  void beginWord ();
  void endWord ();
  void putGaiji (Subbook* subbook, int code, FontType fontType);
  void putChar (TC code, FontType fontType);
  void putString (const String & str);
  void putString (const String & str, FontType fontType);
  void beginGraphic (Subbook* subbook, ImageLink::ImageType type, int width, int height);
  void beginGraphic (Subbook* subbook, ImageLink::ImageType type, const EB_Position & pos);
  void endGraphic ();
  void endGraphic (const EB_Position & pos);
  void beginSound (Subbook* subbook, const EB_Position & pos, int size);
  void endSound ();

 private:
  TextPage * m_page;
  Line *m_line;
  Reference *m_reference;
  Word *m_word;

  void createLine ();
  void addElement (Element * element);
  void putGraphic (Subbook* subbook, const EB_Position & pos);
};

__EBI_END_NAMESPACE

#endif /* __EBI_PAGEMAKER_H */
