/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_reference.h"
#include "ebi_device.h"
#include "ebi_textpage.h"
#include "ebi_book.h"

__EBI_BEGIN_NAMESPACE

Reference::Reference (Subbook* subbook)
{
  m_subbook = subbook;
  m_pos.page = 0;
  m_pos.offset = 0;
}

Reference::~Reference ()
{
}

bool Reference::isValid () const
{
  return (m_pos.page != 0 || m_pos.offset != 0);
}

void Reference::setPos (const EB_Position & pos)
{
  m_pos = pos;
}

Subbook* Reference::getSubbook () const
{
  return m_subbook;
}

void Reference::getPos (EB_Position* pos) const
{
  *pos = m_pos;
}

void Reference::draw (Device* device, W low, W high)
{
  if (isValid ())
    {
      device->beginLink ();
    }

  Container::draw (device, low, high);

  if (isValid ())
    {
      device->endLink ();
    }
}

const Reference* Reference::getReference (const PNT& pnt) const
{
  if (isValid ())
    {
      for (Elements::const_iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
	{
	  if ((*ei)->isHit (pnt))
	    {
	      return this;
	    }
	}
    }
  return NULL;
}

__EBI_END_NAMESPACE
