/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2000-2002  HASEGAWA Masafumi
 */

#include "ebword.h"

// եå
static struct
{
  EB_Hookset hookset;
  BOOL gaiji_code;
}
s_hook;

// Ⱦѳɤ֤
static EB_Error_Code
hook_narrow_font_text (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  char text[80];

  g_error = eb_narrow_alt_character_text (appendix, argv[0], text);
  if (g_error != EB_SUCCESS)
    {
      if (s_hook.gaiji_code)
	{
	  printf (text, "<gaiji:%04x>", argv[0]);
	}
      else
	{
	  strcpy (text, "_");
	}
    }

  eb_write_text_string (book, text);

  return 0;
}

// ѳɤ֤
static EB_Error_Code
hook_wide_font_text (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  char text[80];

  g_error = eb_wide_alt_character_text (appendix, argv[0], text);
  if (g_error != EB_SUCCESS)
    {
      if (s_hook.gaiji_code)
	{
	  printf (text, "<gaiji:%04x>", argv[0]);
	}
      else
	{
	  strcpy (text, "");
	}
    }

  eb_write_text_string (book, text);

  return 0;
}

// GB 2312(Simplified Chinese)֤
static EB_Error_Code
hook_gb2312 (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  char text[80];
  int ku, ten;

  ku  = ((argv[0] >> 8) & 0x7f) - 0x20;
  ten = (argv[0] & 0x7f) - 0x20;
  printf (text, "<gb2312:%02d%02d>", ku, ten);

  eb_write_text_string (book, text);

  return 0;
}

// ISO 8859-1(Latin-1)EUC-JPѴơ֥
const char* c_iso8859_1_to_euc_jp[] = {
  " ",   "_",   "",  "",  "_",   "",  "|",   "",	// a0 - a7
  "",  "(C)", "a",   "",  "",  "",  "(R)", "",	// a8 - af
  "",  "",  "2",   "3",   "",  "",  "",  "",	// b0 - b7
  ",",   "1",   "0",   "",  "1/4", "1/2", "3/4", "_",		// b8 - bf
  "A",   "A",   "A",   "A",   "A",   "A",   "AE",  "C",		// c0 - c7
  "E",   "E",   "E",   "E",   "I",   "I",   "I",   "I",		// c8 - cf
  "D",   "N",   "O",   "O",   "O",   "O",   "O",   "",	// d0 - d7
  "O",   "U",   "U",   "U",   "U",   "Y"  , "P"  , "",	// d8 - df
  "a",   "a",   "a",   "a",   "a",   "a",   "ae",  "c",		// e0 - e7
  "e",   "e",   "e",   "e",   "i",   "i",   "i",   "i",		// e8 - ef
  "o",   "n",   "o",   "o",   "o",   "o",   "o",   "",	// f0 - f7
  "o",   "u",   "u",   "u",   "u",   "y",   "p",   "y"		// f8 - ff
};

// ISO 8859-1(Latin-1)֤
static EB_Error_Code
hook_iso8859_1 (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  const char* text;
  int ch;

  ch = argv[0] & 0xff;
  if (ch & 0x80)
    {
      text = c_iso8859_1_to_euc_jp[(ch & 0x7f) - 0x20];

      eb_write_text_string (book, text);
    }
  else
    {
      eb_write_text_byte1 (book, ch);
    }

  return 0;
}

// Ȥ֤
static EB_Error_Code
hook_reference (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
#if 0
  char text[80];

  printf(text, "<reference:%x:%x>", argv[1], argv[2]);

  eb_write_text_string (book, text);
#endif

  return 0;
}

// Ԥ֤
static EB_Error_Code
hook_newline (EB_Book* book, EB_Appendix* appendix, void* ptr, EB_Hook_Code code, int argc, const unsigned int* argv)
{
  eb_write_text_string (book, "\n");

  return 0;
}

// եåؿΥơ֥
static const EB_Hook s_text_hooks[] = {
  {EB_HOOK_NARROW_FONT, hook_narrow_font_text},
  {EB_HOOK_WIDE_FONT, hook_wide_font_text},
  {EB_HOOK_GB2312, hook_gb2312},
  {EB_HOOK_ISO8859_1, hook_iso8859_1},
  {EB_HOOK_END_REFERENCE, hook_reference},
  {EB_HOOK_NEWLINE, hook_newline},
  {EB_HOOK_NARROW_JISX0208, eb_hook_euc_to_ascii},
  {EB_HOOK_NULL, NULL}
};

// եå
EB_Hookset*
initialize_hookset (BOOL gaiji_code)
{
  s_hook.gaiji_code = gaiji_code;

  eb_initialize_hookset (&s_hook.hookset);
  eb_set_hooks (&s_hook.hookset, s_text_hooks);

  return &s_hook.hookset;
}

// եå
void
terminate_hookset ()
{
  eb_finalize_hookset (&s_hook.hookset);
}
