/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2000-2001  HASEGAWA Masafumi
 */

#include "ebword.h"

EB_Error_Code g_error;

// eb_heading/eb_textδؿɥ쥹
typedef int (*Text_Function) (EB_Book*, EB_Appendix*, EB_Hookset*, void*, size_t, char*, ssize_t*);

// 顼åϤ
void
output_error_message (const char* str)
{
  printf ("%s: %s, %s\n", APP_NAME, str, eb_error_message (g_error));
}

// ʸϤ
static BOOL
output_text (Book* book, EB_Position* position, Text_Function text_function)
{
  char buffer[256];	// Υ礭printf()ưʤʤ
  int len;

  g_error = eb_seek_text (&book->book, position);
  if (g_error != EB_SUCCESS)
    {
      return FALSE;
    }

  while ((g_error = (*text_function) (&book->book, &book->appendix, book->hookset, NULL, sizeof (buffer) - 1, buffer, &len)) == EB_SUCCESS && len > 0)
    {
      buffer[len] = '\0';
      printf (buffer);
    }

  if (g_error != EB_SUCCESS)
    {
      return FALSE;
    }

  return TRUE;
}

// ̤Ϥ
BOOL
output_search (Book* book, BOOL heading_only)
{
  char title[EB_MAX_TITLE_LENGTH + 1];
  EB_Subbook_Code subbook;
  EB_Hit hits[50];
  int hit_count, i;

  eb_subbook (&book->book, &subbook);
  g_error = eb_subbook_title2 (&book->book, subbook, title);
  if (g_error != EB_SUCCESS)
    {
      output_error_message ("ȥǤޤ");
      return FALSE;
    }
  
  while ((g_error = eb_hit_list (&book->book, sizeof (hits) / sizeof (hits[0]), hits, &hit_count)) == EB_SUCCESS && hit_count > 0)
    {
      for (i = 0; i < hit_count; ++i)
	{
	  // Фɽ
	  if (!output_text (book, &(hits[i].heading), eb_read_heading))
	    {
	      output_error_message ("ФǤޤ");
	      return FALSE;
	    }
	  printf (" %s\n", title);
	  if (heading_only)
	    {
	      continue;
	    }

	  // ʸɽ
	  if (!output_text (book, &(hits[i].text), eb_read_text))
	    {
	      output_error_message ("ʸǤޤ");
	      return FALSE;
	    }
	  printf ("\n");
	}
    }

  if (g_error != EB_SUCCESS)
    {
      output_error_message ("ҥåȤȥǤޤ");
      return FALSE;
    }

  return TRUE;
}

// ȥϤ
BOOL
output_title (Book* book, int subbook_index)
{
  char title[EB_MAX_TITLE_LENGTH + 1];

  g_error = eb_subbook_title2 (&book->book, book->subbooks[subbook_index], title);
  if (g_error != EB_SUCCESS)
    {
      output_error_message ("ȥǤޤ");
      return FALSE;
    }

  printf ("%d: %s\n", subbook_index + 1, title);

  return TRUE;
}

// Ϥ
BOOL
output_copyright (Book* book)
{
  EB_Position pos;

  g_error = eb_copyright (&book->book, &pos);
  if (g_error != EB_SUCCESS)
    {
      output_error_message ("Ǥޤ");
      return FALSE;
    }

  if (!output_text (book, &pos, eb_read_text))
    {
      output_error_message ("Ǥޤ");
      return FALSE;
    }
  printf ("\n");

  return TRUE;
}
