/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2000-2001  HASEGAWA Masafumi
 */

#include "ebword.h"
#include "getopt.h"

// СϤ
static void
output_version (void)
{
  printf ("%s, С %s\n", APP_NAME, APP_VERSION);
  printf ("EB饤֥, С %d.%d\n", EB_VERSION_MAJOR, EB_VERSION_MINOR);
}

// ѥ᡼顼Ϥ
static void
output_try_help ()
{
  printf ("%s: ץ󤬰㤤ޤ\n", APP_NAME);
  printf ("ܤ `%s --help' ¹ԤƲ\n", APP_NAME);
}

// ȤϤ
static void
output_usage (void)
{
  printf ("Ȥ: %s [ץ] [ñ]\n", APP_NAME);
  printf ("Ҥñδ׸Ԥ\n");
  printf ("ñκǸ`*'դ׸Ԥ\n");
  printf ("ñκǽ`*'դȸ׸Ԥ\n");
  printf ("\n");
  printf ("  -b ѥ̾, --book=ѥ̾      ҤΥѥ̾ꤹ\n");
  printf ("                                άȸߤκȥեˤʤ\n");
  printf ("  -a ѥ̾, --appendix=ѥ̾  ϿǡΥѥ̾ꤹ\n");
  printf ("                                ά`ҤΥѥ/APPENDIX'ˤʤ\n");
  printf ("  -s ֹ,   --subbook=ֹ     ܤꤹ\n");
  printf ("                                άƤܤ򸡺\n");
  printf ("  -g,        --gaiji            Ͽǡ̵򥳡ɤǽϤ\n");
  printf ("                                ά`_'`'ʸǽϤ\n");
  printf ("  -h,        --heading          ̤θФϤ\n");
  printf ("  -c,        --copyright        ܤϤ\n");
  printf ("  -t,        --title            ܤΥȥ̾Ϥ\n");
  printf ("  -V         --version          СϤ\n");
  printf ("  -H         --help             ȤϤ\n");
}

static const char* c_short_options = "b:a:s:ghctVH";
static const struct option c_long_options[] = {
  {"book", required_argument, NULL, 'b'},
  {"appendix", required_argument, NULL, 'a'},
  {"subbook", required_argument, NULL, 's'},
  {"gaiji", no_argument, NULL, 'g'},
  {"heading", no_argument, NULL, 'h'},
  {"copyright", no_argument, NULL, 'c'},
  {"title", no_argument, NULL, 't'},
  {"version", no_argument, NULL, 'V'},
  {"help", no_argument, NULL, 'H'},
  {NULL, 0, NULL, 0}
};

// ޥɥ饤Ϥ
static BOOL
get_options (int argc, char *const *argv, Params* params)
{
  int ch, remain;

  memset (params, 0, sizeof (Params));
  params->exec_mode = MODE_SEARCH;
  params->subbook_index = ALL_SUBBOOK;

  while ((ch = getopt_long (argc, argv, c_short_options, c_long_options, NULL)) != EOF)
    {
      switch (ch)
	{
	case 'b':
	  strncpy (params->book_path, optarg, L_PATHNM);
	  params->book_path[L_PATHNM] = '\0';
	  break;

	case 'a':
	  strncpy (params->appendix_path, optarg, L_PATHNM);
	  params->appendix_path[L_PATHNM] = '\0';
	  break;

	case 's':
	  params->subbook_index = atoi (optarg) - 1;
	  break;

	case 'g':
	  params->gaiji_code = TRUE;
	  break;

	case 'h':
	  params->heading_only = TRUE;
	  break;

	case 'c':
	  params->exec_mode = MODE_COPYRIGHT;
	  break;

	case 't':
	  params->exec_mode = MODE_TITLE;
	  break;

	case 'V':
	  output_version ();
	  return FALSE;

	case 'H':
	  output_usage ();
	  return FALSE;

	default:
	  output_try_help ();
	  return FALSE;
	}
    }

  remain = argc - optind;
  switch (params->exec_mode)
    {
    case MODE_SEARCH:
      if (remain != 1)
	{
	  output_try_help ();
	  return FALSE;
	}
      strncpy (params->word, argv[optind], EB_MAX_WORD_LENGTH);
      params->word[EB_MAX_WORD_LENGTH] = '\0';
      break;

    default:
      if (remain != 0)
	{
	  output_try_help ();
	  return FALSE;
	}
      break;
    }

  return TRUE;
}

#if !defined(USE_LIBUNIX) && !defined(LINUX)
static char **
tcarg_to_eucarg (W argc, TC *const *tcargv)
{
  char **eucargv;
  int len, i;

  eucargv = calloc (sizeof (char *), argc);
  if (eucargv != NULL)
    {
      for (i = 0; i < argc; ++i)
	{
	  len = tcstoeucs (NULL, tcargv[i]);
	  eucargv[i] = malloc (len + 1);
	  if (eucargv[i] != NULL)
	    {
	      tcstoeucs (eucargv[i], tcargv[i]);
	    }
	}
    }

  return eucargv;
}

static void
free_eucarg (W argc, char **eucargv)
{
  int i;

  if (eucargv != NULL)
    {
      for (i = 0; i < argc; ++i)
	{
	  if (eucargv[i] != NULL)
	    {
	      free (eucargv[i]);
	    }
	}
      free (eucargv);
    }
}
#endif

BOOL
get_params (int argc, char *const *argv, Params* params)
{
#if !defined(USE_LIBUNIX) && !defined(LINUX)
  char **eucargv;
  BOOL ret = FALSE;
  
  eucargv = tcarg_to_eucarg (argc, (TC *const *)argv);
  if (eucargv != NULL)
    {
      ret = get_options (argc, eucargv, params);
      free_eucarg (argc, eucargv);
    }

  return ret;
#else
  return get_options (argc, argv, params);
#endif
}
